@@ -808,7 +808,8 @@ VERSIONED_FIELDS = {
     'secure_boot': versions.MINOR_75_NODE_BOOT_MODE,
     'shard': versions.MINOR_82_NODE_SHARD,
     'parent_node': versions.MINOR_83_PARENT_CHILD_NODES,
-    'firmware_interface': versions.MINOR_86_FIRMWARE_INTERFACE
+    'firmware_interface': versions.MINOR_86_FIRMWARE_INTERFACE,
+    'service_step': versions.MINOR_87_SERVICE
 }
 
 for field in V31_FIELDS:
@@ -1655,7 +1656,8 @@ def check_port_policy_and_retrieve(policy_name, port_ident, portgroup=False):
                         a port or portgroup by.
 
     :raises: HTTPForbidden if the policy forbids access.
-    :raises: NodeNotFound if the node is not found.
+    :raises: PortNotFound if the port is not found.
+    :raises: PortgroupNotFound if the portgroup is not found.
     :return: RPC port identified by port_ident associated node
     """
     context = api.request.context
@@ -1957,6 +1959,11 @@ def allow_unhold_verb():
     return api.request.version.minor >= versions.MINOR_85_UNHOLD_VERB
 
 
+def allow_service_verb():
+    """Check if the service verb may be passed to the API."""
+    return api.request.version.minor >= versions.MINOR_87_SERVICE
+
+
 def check_allow_deploy_steps(target, deploy_steps):
     """Check if deploy steps are allowed"""
 
@@ -2012,6 +2019,19 @@ def allow_continue_inspection_endpoint():
 def allow_firmware_interface():
     """Check if we should support firmware interface and endpoints.
 
-    Version 1.84 of the API added support for firmware interface.
+    Version 1.86 of the API added support for firmware interface.
     """
     return api.request.version.minor >= versions.MINOR_86_FIRMWARE_INTERFACE
+
+
+def allow_port_name():
+    """Check if name is allowed for ports.
+
+    Version 1.88 of the API added name field to the port object.
+    """
+    return api.request.version.minor >= versions.MINOR_88_PORT_NAME
+
+
+def allow_attach_detach_vmedia():
+    """Check if we should support virtual media actions."""
+    return api.request.version.minor >= versions.MINOR_89_ATTACH_DETACH_VMEDIA
