@@ -21,17 +21,20 @@ Handling of VM disk images.
 
 import os
 import shutil
+import time
 
-from ironic_lib import disk_utils
-from ironic_lib import utils as ironic_utils
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import fileutils
+import pycdlib
 
+from ironic.common import checksum_utils
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
 from ironic.common.i18n import _
+from ironic.common import image_format_inspector
 from ironic.common import image_service as service
+from ironic.common import qemu_img
 from ironic.common import utils
 from ironic.conf import CONF
 
@@ -72,14 +75,6 @@ def _create_root_fs(root_directory, files_info):
             shutil.copyfile(src_file, target_file)
 
 
-def _umount_without_raise(mount_dir):
-    """Helper method to umount without raise."""
-    try:
-        utils.umount(mount_dir)
-    except processutils.ProcessExecutionError:
-        pass
-
-
 def create_vfat_image(output_file, files_info=None, parameters=None,
                       parameters_file='parameters.txt', fs_size_kib=100):
     """Creates the fat fs image on the desired file.
@@ -106,8 +101,9 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
              mounting, creating filesystem, copying files, etc.
     """
     try:
-        ironic_utils.dd('/dev/zero', output_file, 'count=1',
-                        "bs=%dKiB" % fs_size_kib)
+        # TODO(sbaker): use ironic_lib.utils.dd when rootwrap has been removed
+        utils.execute('dd', 'if=/dev/zero', 'of=%s' % output_file, 'count=1',
+                      'bs=%dKiB' % fs_size_kib)
     except processutils.ProcessExecutionError as e:
         raise exception.ImageCreationFailed(image_type='vfat', error=e)
 
@@ -117,8 +113,10 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
             # The label helps ramdisks to find the partition containing
             # the parameters (by using /dev/disk/by-label/ir-vfd-dev).
             # NOTE: FAT filesystem label can be up to 11 characters long.
-            ironic_utils.mkfs('vfat', output_file, label="ir-vfd-dev")
-            utils.mount(output_file, tmpdir, '-o', 'umask=0')
+            # TODO(sbaker): use ironic_lib.utils.mkfs when rootwrap has been
+            #              removed
+            utils.execute('mkfs', '-t', 'vfat', '-n',
+                          'ir-vfd-dev', output_file)
         except processutils.ProcessExecutionError as e:
             raise exception.ImageCreationFailed(image_type='vfat', error=e)
 
@@ -133,16 +131,21 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
                 file_contents = '\n'.join(params_list)
                 utils.write_to_file(parameters_file, file_contents)
 
+            file_list = os.listdir(tmpdir)
+
+            if not file_list:
+                return
+
+            file_list = [os.path.join(tmpdir, item) for item in file_list]
+
+            # use mtools to copy the files into the image in a single
+            # operation
+            utils.execute('mcopy', '-s', *file_list, '-i', output_file, '::')
+
         except Exception as e:
             LOG.exception("vfat image creation failed. Error: %s", e)
             raise exception.ImageCreationFailed(image_type='vfat', error=e)
 
-        finally:
-            try:
-                utils.umount(tmpdir)
-            except processutils.ProcessExecutionError as e:
-                raise exception.ImageCreationFailed(image_type='vfat', error=e)
-
 
 def _generate_cfg(kernel_params, template, options):
     """Generates a isolinux or grub configuration file.
@@ -238,7 +241,7 @@ def create_isolinux_image_for_bios(
 
         try:
             utils.execute('mkisofs', '-r', '-V', _label(files_info),
-                          '-cache-inodes', '-J', '-l', '-no-emul-boot',
+                          '-J', '-l', '-no-emul-boot',
                           '-boot-load-size', '4', '-boot-info-table',
                           '-b', ISOLINUX_BIN, '-o', output_file, tmpdir)
         except processutils.ProcessExecutionError as e:
@@ -291,7 +294,7 @@ def create_esp_image_for_uefi(
             # directory.
             if deploy_iso and not esp_image:
                 uefi_path_info, e_img_rel_path, grub_rel_path = (
-                    _mount_deploy_iso(deploy_iso, mountdir))
+                    _get_deploy_iso_files(deploy_iso, mountdir))
 
                 grub_cfg = os.path.join(tmpdir, grub_rel_path)
 
@@ -328,7 +331,7 @@ def create_esp_image_for_uefi(
 
             finally:
                 if deploy_iso:
-                    _umount_without_raise(mountdir)
+                    shutil.rmtree(mountdir)
 
         # Generate and copy grub config file.
         grub_conf = _generate_cfg(kernel_params,
@@ -346,7 +349,7 @@ def create_esp_image_for_uefi(
             raise exception.ImageCreationFailed(image_type='iso', error=e)
 
 
-def fetch(context, image_href, path, force_raw=False):
+def fetch_into(context, image_href, image_file):
     # TODO(vish): Improve context handling and add owner and auth data
     #             when it is added to glance.  Right now there is no
     #             auth checking in glance, so we assume that access was
@@ -354,34 +357,39 @@ def fetch(context, image_href, path, force_raw=False):
     image_service = service.get_image_service(image_href,
                                               context=context)
     LOG.debug("Using %(image_service)s to download image %(image_href)s.",
-              {'image_service': image_service.__class__,
+              {'image_service': image_service.__class__.__name__,
                'image_href': image_href})
+    start = time.time()
 
+    if isinstance(image_file, str):
+        with open(image_file, "wb") as image_file_obj:
+            image_service.download(image_href, image_file_obj)
+    else:
+        image_service.download(image_href, image_file)
+
+    LOG.debug("Image %(image_href)s downloaded in %(time).2f seconds.",
+              {'image_href': image_href, 'time': time.time() - start})
+
+
+def fetch(context, image_href, path, force_raw=False,
+          checksum=None, checksum_algo=None):
     with fileutils.remove_path_on_error(path):
-        with open(path, "wb") as image_file:
-            image_service.download(image_href, image_file)
-
+        fetch_into(context, image_href, path)
+        if (not CONF.conductor.disable_file_checksum
+                and checksum):
+            checksum_utils.validate_checksum(path, checksum, checksum_algo)
     if force_raw:
         image_to_raw(image_href, path, "%s.part" % path)
 
 
 def get_source_format(image_href, path):
-    data = disk_utils.qemu_img_info(path)
-
-    fmt = data.file_format
-    if fmt is None:
+    try:
+        img_format = image_format_inspector.detect_file_format(path)
+    except image_format_inspector.ImageFormatError:
         raise exception.ImageUnacceptable(
-            reason=_("'qemu-img info' parsing failed."),
+            reason=_("parsing of the image failed."),
             image_id=image_href)
-
-    backing_file = data.backing_file
-    if backing_file is not None:
-        raise exception.ImageUnacceptable(
-            image_id=image_href,
-            reason=_("fmt=%(fmt)s backed by: %(backing_file)s") %
-            {'fmt': fmt, 'backing_file': backing_file})
-
-    return fmt
+    return str(img_format)
 
 
 def force_raw_will_convert(image_href, path_tmp):
@@ -394,22 +402,46 @@ def force_raw_will_convert(image_href, path_tmp):
 
 def image_to_raw(image_href, path, path_tmp):
     with fileutils.remove_path_on_error(path_tmp):
-        fmt = get_source_format(image_href, path_tmp)
+        if not CONF.conductor.disable_deep_image_inspection:
+            fmt = safety_check_image(path_tmp)
 
-        if fmt != "raw":
+            if fmt not in CONF.conductor.permitted_image_formats:
+                LOG.error("Security: The requested image %(image_href)s "
+                          "is of format image %(format)s and is not in "
+                          "the [conductor]permitted_image_formats list.",
+                          {'image_href': image_href,
+                           'format': fmt})
+                raise exception.InvalidImage()
+        else:
+            fmt = get_source_format(image_href, path)
+            LOG.warning("Security: Image safety checking has been disabled. "
+                        "This is unsafe operation. Attempting to continue "
+                        "the detected format %(img_fmt)s for %(path)s.",
+                        {'img_fmt': fmt,
+                         'path': path})
+
+        if fmt != "raw" and fmt != "iso":
+            # When the target format is NOT raw, we need to convert it.
+            # however, we don't need nor want to do that when we have
+            # an ISO image. If we have an ISO because it was requested,
+            # we have correctly fingerprinted it. Prior to proper
+            # image detection, we thought we had a raw image, and we
+            # would end up asking for a raw image to be made a raw image.
             staged = "%s.converted" % path
+
+            utils.is_memory_insufficient(raise_if_fail=True)
             LOG.debug("%(image)s was %(format)s, converting to raw",
                       {'image': image_href, 'format': fmt})
             with fileutils.remove_path_on_error(staged):
-                disk_utils.convert_image(path_tmp, staged, 'raw')
+                qemu_img.convert_image(path_tmp, staged, 'raw',
+                                       source_format=fmt)
                 os.unlink(path_tmp)
-
-                data = disk_utils.qemu_img_info(staged)
-                if data.file_format != "raw":
+                new_fmt = get_source_format(image_href, staged)
+                if new_fmt != "raw":
                     raise exception.ImageConvertFailed(
                         image_id=image_href,
                         reason=_("Converted to raw, but format is "
-                                 "now %s") % data.file_format)
+                                 "now %s") % new_fmt)
 
                 os.rename(staged, path)
         else:
@@ -426,18 +458,25 @@ def download_size(context, image_href, image_service=None):
     return image_show(context, image_href, image_service)['size']
 
 
-def converted_size(path):
+def converted_size(path, estimate=False):
     """Get size of converted raw image.
 
     The size of image converted to raw format can be growing up to the virtual
     size of the image.
 
     :param path: path to the image file.
-    :returns: virtual size of the image or 0 if conversion not needed.
-
+    :param estimate: Whether to estimate the size by scaling the
+        original size
+    :returns: For `estimate=False`, return the size of the
+        raw image file. For `estimate=True`, return the size of
+        the original image scaled by the configuration value
+        `raw_image_growth_factor`.
     """
-    data = disk_utils.qemu_img_info(path)
-    return data.virtual_size
+    data = image_format_inspector.detect_file_format(path)
+    if not estimate:
+        return data.virtual_size
+    growth_factor = CONF.raw_image_growth_factor
+    return int(min(data.actual_size * growth_factor, data.virtual_size))
 
 
 def get_image_properties(context, image_href, properties="all"):
@@ -477,7 +516,7 @@ def get_temp_url_for_glance_image(context, image_uuid):
 def create_boot_iso(context, output_filename, kernel_href,
                     ramdisk_href, deploy_iso_href=None, esp_image_href=None,
                     root_uuid=None, kernel_params=None, boot_mode=None,
-                    base_iso=None, inject_files=None):
+                    inject_files=None):
     """Creates a bootable ISO image for a node.
 
     Given the hrefs for kernel, ramdisk, root partition's UUID and
@@ -501,28 +540,15 @@ def create_boot_iso(context, output_filename, kernel_href,
     :param kernel_params: a string containing whitespace separated values
         kernel cmdline arguments of the form K=V or K (optional).
     :boot_mode: the boot mode in which the deploy is to happen.
-    :param base_iso: URL or glance UUID of a to be used as an override of
-        what should be retrieved for to use, instead of building an ISO
-        bootable ramdisk.
     :param inject_files: Mapping of local source file paths to their location
         on the final ISO image.
     :raises: ImageCreationFailed, if creating boot ISO failed.
     """
     with utils.tempdir() as tmpdir:
-        if base_iso:
-            # NOTE(TheJulia): Eventually we want to use the creation method
-            # to perform the massaging of the image, because oddly enough
-            # we need to do all the same basic things, just a little
-            # differently.
-            fetch(context, base_iso, output_filename)
-            # Temporary, return to the caller until we support the combined
-            # operation.
-            return
-        else:
-            kernel_path = os.path.join(tmpdir, kernel_href.split('/')[-1])
-            ramdisk_path = os.path.join(tmpdir, ramdisk_href.split('/')[-1])
-            fetch(context, kernel_href, kernel_path)
-            fetch(context, ramdisk_href, ramdisk_path)
+        kernel_path = os.path.join(tmpdir, 'kernel')
+        ramdisk_path = os.path.join(tmpdir, 'ramdisk')
+        fetch(context, kernel_href, kernel_path)
+        fetch(context, ramdisk_href, ramdisk_path)
 
         params = []
         if root_uuid:
@@ -535,13 +561,11 @@ def create_boot_iso(context, output_filename, kernel_href,
             deploy_iso_path = esp_image_path = None
 
             if deploy_iso_href:
-                deploy_iso_path = os.path.join(
-                    tmpdir, deploy_iso_href.split('/')[-1])
+                deploy_iso_path = os.path.join(tmpdir, 'iso')
                 fetch(context, deploy_iso_href, deploy_iso_path)
 
             elif esp_image_href:
-                esp_image_path = os.path.join(
-                    tmpdir, esp_image_href.split('/')[-1])
+                esp_image_path = os.path.join(tmpdir, 'esp')
                 fetch(context, esp_image_href, esp_image_path)
 
             elif CONF.esp_image:
@@ -562,6 +586,11 @@ def create_boot_iso(context, output_filename, kernel_href,
                 kernel_params=params, inject_files=inject_files)
 
 
+IMAGE_TYPE_PARTITION = 'partition'
+IMAGE_TYPE_WHOLE_DISK = 'whole-disk'
+VALID_IMAGE_TYPES = frozenset((IMAGE_TYPE_PARTITION, IMAGE_TYPE_WHOLE_DISK))
+
+
 def is_whole_disk_image(ctx, instance_info):
     """Find out if the image is a partition image or a whole disk image.
 
@@ -569,22 +598,38 @@ def is_whole_disk_image(ctx, instance_info):
     :param instance_info: a node's instance info dict
 
     :returns: True for whole disk images and False for partition images
-        and None on no image_source or Error.
+        and None on no image_source, the source being a path, or upon an
+        Error.
     """
     image_source = instance_info.get('image_source')
     if not image_source:
         return
 
+    image_type = instance_info.get('image_type')
+    if image_type:
+        # This logic reflects the fact that whole disk images are the default
+        return image_type != IMAGE_TYPE_PARTITION
+
     is_whole_disk_image = False
     if glance_utils.is_glance_image(image_source):
         try:
             iproperties = get_image_properties(ctx, image_source)
         except Exception:
             return
+
+        image_type = iproperties.get('img_type')
+        if image_type:
+            return image_type != IMAGE_TYPE_PARTITION
+
         is_whole_disk_image = (not iproperties.get('kernel_id')
                                and not iproperties.get('ramdisk_id'))
     else:
         # Non glance image ref
+        if is_source_a_path(ctx, instance_info.get('image_source')):
+            # Nothing is returned if not valid or there was an error.
+            # A third possibility is it is not a disk image, which would
+            # still be None.
+            return
         if (not instance_info.get('kernel')
             and not instance_info.get('ramdisk')):
             is_whole_disk_image = True
@@ -592,12 +637,103 @@ def is_whole_disk_image(ctx, instance_info):
     return is_whole_disk_image
 
 
-def _mount_deploy_iso(deploy_iso, mountdir):
+def is_source_a_path(ctx, image_source):
+    """Determine if the image source is a path.
+
+    This method determines if a supplied URL is a path.
+
+    :param ctx: an admin/process context.
+    :param image_source: The supplied image source, expected to be a
+                         URL, which can be used to attempt to determine
+                         if the source is a path.
+    :returns: True if the image_source appears to be a path as opposed
+              to an image to be downloaded. If the image source is not
+              a path, False is returned. If any error is detected,
+              None is returned.
+    """
+    if not image_source:
+        return
+    image_service = service.get_image_service(image_source,
+                                              context=ctx)
+    try:
+        res = image_service.validate_href(image_source)
+        if 'headers' in dir(res):
+            # response/result is from the HTTP check path.
+            headers = res.headers
+        else:
+            # We have no headers.
+            headers = {}
+    except exception.ImageRefIsARedirect as e:
+        # Our exception handling formats this for us in this
+        # case. \o/
+        LOG.debug(str(e))
+        # Servers redirect to a proper folder ending in a slash if
+        # not supplied originally.
+        if e.redirect_url and e.redirect_url.endswith('/'):
+            return True
+    except Exception:
+        # NOTE(TheJulia): I don't really like this pattern, *but*
+        # the wholedisk image support is similar.
+        return
+    # NOTE(TheJulia): Files should have been caught almost exclusively
+    # before with the Content-Length check.
+    # When the ISO is mounted and the webserver mount point url is
+    # checked here, it has both 'Content-Length' and 'Content-Type'
+    # due to which it always returns False. Hence switched the conditions.
+    if ('Content-Type' in headers
+            and str(headers['Content-Type']).startswith('text/html')):
+        LOG.debug('Evaluated %(url)s to determine if it is a URL to a path '
+                  'or a file. A Content-Type header was returned with a text '
+                  'content, which suggests a file list was returned.',
+                  {'url': image_source})
+        return True
+    # When issuing a head request, folders have no length
+    # A list can be generated by the server.. This is a solid
+    # hint.
+    if ('Content-Type' in headers
+            and (str(headers['Content-Type']) != 'text/html')
+            and 'Content-Length' in headers):
+        LOG.debug('Evaluated %(url)s to determine if it is a URL to a path '
+                  'or a file. A Content-Length header was returned '
+                  'suggesting file.',
+                  {'url': image_source})
+        # NOTE(TheJulia): Files on a webserver have a length which is returned
+        # when headres are queried.
+        return False
+    if image_source.endswith('/'):
+        # If all else fails, looks like a URL, and the server didn't give
+        # us any hints.
+        return True
+    # We were unable to determine if this was a folder or a file.
+    return False
+
+
+def _extract_iso(extract_iso, extract_dir):
+    # NOTE(rpittau): we could probably just extract the files we need
+    # if we find them. Also we probably need to detect the correct iso
+    # type (UDF, RR, JOLIET).
+    iso = pycdlib.PyCdlib()
+    iso.open(extract_iso)
+
+    for dirname, dirlist, filelist in iso.walk(iso_path='/'):
+        dir_path = dirname.lstrip('/')
+        for dir_iso in dirlist:
+            os.makedirs(os.path.join(extract_dir, dir_path, dir_iso))
+        for file in filelist:
+            file_path = os.path.join(extract_dir, dirname, file)
+            iso.get_file_from_iso(
+                os.path.join(extract_dir, dir_path, file),
+                iso_path=file_path)
+
+    iso.close()
+
+
+def _get_deploy_iso_files(deploy_iso, mountdir):
     """This function opens up the deploy iso used for deploy.
 
     :param deploy_iso: path to the deploy iso where its
                        contents are fetched to.
-    :raises: ImageCreationFailed if mount fails.
+    :raises: ImageCreationFailed if extraction fails.
     :returns: a tuple consisting of - 1. a dictionary containing
                                          the values as required
                                          by create_isolinux_image,
@@ -611,9 +747,9 @@ def _mount_deploy_iso(deploy_iso, mountdir):
     grub_path = None
 
     try:
-        utils.mount(deploy_iso, mountdir, '-o', 'loop')
-    except processutils.ProcessExecutionError as e:
-        LOG.exception("mounting the deploy iso failed.")
+        _extract_iso(deploy_iso, mountdir)
+    except Exception as e:
+        LOG.exception("extracting the deploy iso failed.")
         raise exception.ImageCreationFailed(image_type='iso', error=e)
 
     try:
@@ -628,14 +764,14 @@ def _mount_deploy_iso(deploy_iso, mountdir):
                                                 mountdir)
     except (OSError, IOError) as e:
         LOG.exception("examining the deploy iso failed.")
-        _umount_without_raise(mountdir)
+        shutil.rmtree(mountdir)
         raise exception.ImageCreationFailed(image_type='iso', error=e)
 
     # check if the variables are assigned some values or not during
     # walk of the mountdir.
     if not (e_img_path and e_img_rel_path and grub_path and grub_rel_path):
         error = (_("Deploy iso didn't contain efiboot.img or grub.cfg"))
-        _umount_without_raise(mountdir)
+        shutil.rmtree(mountdir)
         raise exception.ImageCreationFailed(image_type='iso', error=error)
 
     uefi_path_info = {e_img_path: e_img_rel_path,
@@ -649,3 +785,92 @@ def _mount_deploy_iso(deploy_iso, mountdir):
     # present in deploy iso. This path varies for different OS vendors.
     # e_img_rel_path: is required by mkisofs to generate boot iso.
     return uefi_path_info, e_img_rel_path, grub_rel_path
+
+
+def __node_or_image_cache(node):
+    """A helper for logging to determine if image cache or node uuid."""
+    if not node:
+        return 'image cache'
+    else:
+        return node.uuid
+
+
+def safety_check_image(image_path, node=None):
+    """Performs a safety check on the supplied image.
+
+    This method triggers the image format inspector's to both identify the
+    type of the supplied file and safety check logic to identify if there
+    are any known unsafe features being leveraged, and return the detected
+    file format in the form of a string for the caller.
+
+    :param image_path: A fully qualified path to an image which needs to
+                       be evaluated for safety.
+    :param node: A Node object, optional. When supplied logging indicates the
+                 node which triggered this issue, but the node is not
+                 available in all invocation cases.
+    :returns: a string representing the the image type which is used.
+    :raises: InvalidImage when the supplied image is detected as unsafe,
+             or the image format inspector has failed to parse the supplied
+             image's contents.
+    """
+    id_string = __node_or_image_cache(node)
+    try:
+        img_class = image_format_inspector.detect_file_format(image_path)
+        if not img_class.safety_check():
+            LOG.error("Security: The requested image for "
+                      "deployment of node %(node)s fails safety sanity "
+                      "checking.",
+                      {'node': id_string})
+            raise exception.InvalidImage()
+        image_format_name = str(img_class)
+    except image_format_inspector.ImageFormatError:
+        LOG.error("Security: The requested user image for the "
+                  "deployment node %(node)s failed to be able "
+                  "to be parsed by the image format checker.",
+                  {'node': id_string})
+        raise exception.InvalidImage()
+    return image_format_name
+
+
+def check_if_image_format_is_permitted(img_format,
+                                       expected_format=None,
+                                       node=None):
+    """Checks image format consistency.
+
+    :params img_format: The determined image format by name.
+    :params expected_format: Optional, the expected format based upon
+        supplied configuration values.
+    :params node: A node object or None implying image cache.
+    :raises: InvalidImage if the requested image format is not permitted
+             by configuration, or the expected_format does not match the
+             determined format.
+    """
+
+    id_string = __node_or_image_cache(node)
+    if img_format not in CONF.conductor.permitted_image_formats:
+        LOG.error("Security: The requested deploy image for node %(node)s "
+                  "is of format image %(format)s and is not in the "
+                  "[conductor]permitted_image_formats list.",
+                  {'node': id_string,
+                   'format': img_format})
+        raise exception.InvalidImage()
+    if expected_format is not None and img_format != expected_format:
+        if expected_format in ['ari', 'aki']:
+            # In this case, we have an ari or aki, meaning we're pulling
+            # down a kernel/ramdisk, and this is rooted in a misunderstanding.
+            # They should be raw. The detector should be detecting this *as*
+            # raw anyway, so the data just mismatches from a common
+            # misunderstanding, and that is okay in this case as they are not
+            # passed to qemu-img.
+            # TODO(TheJulia): Add a log entry to warn here at some point in
+            # the future as we begin to shift the perception around this.
+            # See: https://bugs.launchpad.net/ironic/+bug/2074090
+            return
+        LOG.error("Security: The requested deploy image for node %(node)s "
+                  "has a format (%(format)s) which does not match the "
+                  "expected image format (%(expected)s) based upon "
+                  "supplied or retrieved information.",
+                  {'node': id_string,
+                   'format': img_format,
+                   'expected': expected_format})
+        raise exception.InvalidImage()
