@@ -16,7 +16,8 @@ import ipaddress
 import openstack
 from openstack.connection import exceptions as openstack_exc
 from oslo_log import log
-import retrying
+import tenacity
+from tenacity import retry
 
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.common import context as ironic_context
@@ -69,7 +70,11 @@ def get_client(token=None, context=None, auth_from_config=False):
 
     user_auth = None
     if (not auth_from_config and CONF.neutron.auth_type != 'none'
-            and context.auth_token):
+            and context.auth_token and not context.system_scope):
+        # If we have a token, we *should* use the user's auth, however we
+        # can only do so *if* it is a project scoped request. If it is
+        # system scoped, we cannot leverage user auth data to make the next
+        # request.
         user_auth = keystone.get_service_auth(context, endpoint, service_auth)
 
     sess = keystone.get_session('neutron', timeout=CONF.neutron.timeout,
@@ -344,7 +349,8 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
                 wait_for_host_agent(
                     client, update_port_attrs['binding:host_id'])
             port = client.create_port(**port_attrs)
-            update_neutron_port(task.context, port.id, update_port_attrs)
+            port = update_neutron_port(task.context, port.id,
+                                       update_port_attrs)
             if CONF.neutron.dhcpv6_stateful_address_count > 1:
                 _add_ip_addresses_for_ipv6_stateful(task.context, port, client)
             if is_smart_nic:
@@ -711,7 +717,7 @@ def validate_port_info(node, port):
     # Subnet Manager.
     if port.extra.get('client-id'):
         return True
-    if (node.network_interface == 'neutron'
+    if (node.get_interface('network') == 'neutron'
             and not port.local_link_connection):
         LOG.warning("The local_link_connection is required for "
                     "'neutron' network interface and is not present "
@@ -865,11 +871,11 @@ def get_physnets_by_port_uuid(client, port_uuid):
                 if network.provider_physical_network else set())
 
 
-@retrying.retry(
-    stop_max_attempt_number=CONF.agent.neutron_agent_max_attempts,
-    retry_on_exception=lambda e: isinstance(e, exception.NetworkError),
-    wait_fixed=CONF.agent.neutron_agent_status_retry_interval * 1000
-)
+@retry(
+    retry=tenacity.retry_if_exception_type(exception.NetworkError),
+    stop=tenacity.stop_after_attempt(CONF.agent.neutron_agent_max_attempts),
+    wait=tenacity.wait_fixed(CONF.agent.neutron_agent_status_retry_interval),
+    reraise=True)
 def wait_for_host_agent(client, host_id, target_state='up'):
     """Wait for neutron agent to become target state
 
@@ -904,11 +910,11 @@ def wait_for_host_agent(client, host_id, target_state='up'):
             'host': host_id, 'state': target_state})
 
 
-@retrying.retry(
-    stop_max_attempt_number=CONF.agent.neutron_agent_max_attempts,
-    retry_on_exception=lambda e: isinstance(e, exception.NetworkError),
-    wait_fixed=CONF.agent.neutron_agent_status_retry_interval * 1000
-)
+@retry(
+    retry=tenacity.retry_if_exception_type(exception.NetworkError),
+    stop=tenacity.stop_after_attempt(CONF.agent.neutron_agent_max_attempts),
+    wait=tenacity.wait_fixed(CONF.agent.neutron_agent_status_retry_interval),
+    reraise=True)
 def wait_for_port_status(client, port_id, status):
     """Wait for port status to be the desired status
 
@@ -987,3 +993,12 @@ class NeutronNetworkInterfaceMixin(object):
             # Fall back to non-managed in-band inspection
             raise exception.UnsupportedDriverExtension(
                 driver=task.node.driver, extension='inspection')
+
+    def get_servicing_network_uuid(self, task):
+        servicing_network = (
+            task.node.driver_info.get('servicing_network')
+            or CONF.neutron.servicing_network
+        )
+        return validate_network(
+            servicing_network, _('servicing network'),
+            context=task.context)
