@@ -16,7 +16,6 @@
 
 import os
 import re
-import time
 
 from ironic_lib import metrics_utils
 from ironic_lib import utils as il_utils
@@ -25,6 +24,8 @@ from oslo_utils import excutils
 from oslo_utils import fileutils
 from oslo_utils import strutils
 
+from ironic.common import checksum_utils
+from ironic.common import context
 from ironic.common import exception
 from ironic.common import faults
 from ironic.common.glance_service import service_utils
@@ -41,24 +42,14 @@ from ironic.drivers.modules import image_cache
 from ironic.drivers import utils as driver_utils
 from ironic import objects
 
-# TODO(Faizan): Move this logic to common/utils.py and deprecate
-# rootwrap_config.
-# This is required to set the default value of ironic_lib option
-# only if rootwrap_config does not contain the default value.
-
-if CONF.rootwrap_config != '/etc/ironic/rootwrap.conf':
-    root_helper = 'sudo ironic-rootwrap %s' % CONF.rootwrap_config
-    CONF.set_default('root_helper', root_helper, 'ironic_lib')
 
 LOG = logging.getLogger(__name__)
 
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
 SUPPORTED_CAPABILITIES = {
-    'boot_option': ('local', 'netboot', 'ramdisk', 'kickstart'),
     'boot_mode': ('bios', 'uefi'),
     'secure_boot': ('true', 'false'),
-    'trusted_boot': ('true', 'false'),
     'disk_label': ('msdos', 'gpt'),
 }
 
@@ -68,6 +59,7 @@ RESCUE_LIKE_STATES = (states.RESCUING, states.RESCUEWAIT, states.RESCUEFAIL,
 
 DISK_LAYOUT_PARAMS = ('root_gb', 'swap_mb', 'ephemeral_gb')
 
+
 # All functions are called from deploy() directly or indirectly.
 # They are split for stub-out.
 
@@ -130,13 +122,10 @@ def _replace_root_uuid(path, root_uuid):
 
 
 def _replace_boot_line(path, boot_mode, is_whole_disk_image,
-                       trusted_boot=False, iscsi_boot=False,
-                       ramdisk_boot=False, ipxe_enabled=False,
-                       anaconda_boot=False):
+                       iscsi_boot=False, ramdisk_boot=False,
+                       ipxe_enabled=False, anaconda_boot=False):
     if is_whole_disk_image:
         boot_disk_type = 'boot_whole_disk'
-    elif trusted_boot:
-        boot_disk_type = 'trusted_boot'
     elif iscsi_boot:
         boot_disk_type = 'boot_iscsi'
     elif ramdisk_boot:
@@ -163,10 +152,13 @@ def _replace_disk_identifier(path, disk_identifier):
 
 
 # NOTE(TheJulia): This should likely be migrated to pxe_utils.
+# TODO(dtantsur): with the removal of netboot, root_uuid_or_disk_id and
+# the logic of replacing ROOT can be dropped, while is_whole_disk_image can
+# be renamed to something like netboot_fallback.
 def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
-                      is_whole_disk_image, trusted_boot=False,
-                      iscsi_boot=False, ramdisk_boot=False,
-                      ipxe_enabled=False, anaconda_boot=False):
+                      is_whole_disk_image, iscsi_boot=False,
+                      ramdisk_boot=False, ipxe_enabled=False,
+                      anaconda_boot=False):
     """Switch a pxe config from deployment mode to service mode.
 
     :param path: path to the pxe config file in tftpboot.
@@ -174,9 +166,6 @@ def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
                                  disk_id in case of whole disk image.
     :param boot_mode: if boot mode is uefi or bios.
     :param is_whole_disk_image: if the image is a whole disk image or not.
-    :param trusted_boot: if boot with trusted_boot or not. The usage of
-        is_whole_disk_image and trusted_boot are mutually exclusive. You can
-        have one or neither, but not both.
     :param iscsi_boot: if boot is from an iSCSI volume or not.
     :param ramdisk_boot: if the boot is to be to a ramdisk configuration.
     :param ipxe_enabled: A default False boolean value to tell the method
@@ -190,8 +179,8 @@ def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
         else:
             _replace_disk_identifier(path, root_uuid_or_disk_id)
 
-    _replace_boot_line(path, boot_mode, is_whole_disk_image, trusted_boot,
-                       iscsi_boot, ramdisk_boot, ipxe_enabled, anaconda_boot)
+    _replace_boot_line(path, boot_mode, is_whole_disk_image, iscsi_boot,
+                       ramdisk_boot, ipxe_enabled, anaconda_boot)
 
 
 def check_for_missing_params(info_dict, error_msg, param_prefix=''):
@@ -216,7 +205,9 @@ def check_for_missing_params(info_dict, error_msg, param_prefix=''):
                        'missing_info': missing_info})
 
 
-def fetch_images(ctx, cache, images_info, force_raw=True):
+def fetch_images(ctx, cache, images_info, force_raw=True,
+                 expected_format=None, expected_checksum=None,
+                 expected_checksum_algo=None):
     """Check for available disk space and fetch images using ImageCache.
 
     :param ctx: context
@@ -224,7 +215,15 @@ def fetch_images(ctx, cache, images_info, force_raw=True):
     :param images_info: list of tuples (image href, destination path)
     :param force_raw: boolean value, whether to convert the image to raw
                       format
+    :param expected_format: The expected format of the image.
+    :param expected_checksum: The expected image checksum, to be used if we
+           need to convert the image to raw prior to deploying.
+    :param expected_checksum_algo: The checksum algo in use, if separately
+           set.
     :raises: InstanceDeployFailure if unable to find enough disk space
+    :raises: InvalidImage if the supplied image metadata or contents are
+             deemed to be invalid, unsafe, or not matching the expectations
+             asserted by configuration supplied or set.
     """
 
     try:
@@ -236,8 +235,17 @@ def fetch_images(ctx, cache, images_info, force_raw=True):
     # if disk space is used between the check and actual download.
     # This is probably unavoidable, as we can't control other
     # (probably unrelated) processes
+    image_list = []
     for href, path in images_info:
-        cache.fetch_image(href, path, ctx=ctx, force_raw=force_raw)
+        # NOTE(TheJulia): Href in this case can be an image UUID or a URL.
+        image_format = cache.fetch_image(
+            href, path, ctx=ctx,
+            force_raw=force_raw,
+            expected_format=expected_format,
+            expected_checksum=expected_checksum,
+            expected_checksum_algo=expected_checksum_algo)
+        image_list.append((href, path, image_format))
+    return image_list
 
 
 def set_failed_state(task, msg, collect_logs=True):
@@ -561,8 +569,10 @@ def validate_image_properties(task, deploy_info):
     :raises: MissingParameterValue if the image doesn't contain
         the mentioned properties.
     """
+    node = task.node
     image_href = deploy_info.get('image_source')
     boot_iso = deploy_info.get('boot_iso')
+    isap = task.node.driver_internal_info.get('is_source_a_path')
     if image_href and boot_iso:
         raise exception.InvalidParameterValue(_(
             "An 'image_source' and 'boot_iso' parameter may not be "
@@ -573,15 +583,22 @@ def validate_image_properties(task, deploy_info):
     boot_option = get_boot_option(task.node)
 
     if (boot_iso
-            or task.node.driver_internal_info.get('is_whole_disk_image')
-            or boot_option == 'local'):
+            or node.driver_internal_info.get('is_whole_disk_image')
+            or boot_option == 'local'
+            or isap):
         # No image properties are required in this case, but validate that the
         # image at least looks reasonable.
         try:
+            # This doesn't actually test *getting* the defined url or file
+            # but actually validates we can parse the data *to* connect.
             image_service.get_image_service(image_href, context=task.context)
         except exception.ImageRefValidationFailed as e:
             raise exception.InvalidParameterValue(err=e)
-        return
+        if not isap:
+            # If the source is not a path, but otherwise matches, we need to
+            # exit validation here. Deployments, such as ramdisk or anaconda
+            # need further parameter validation and this supplies it.
+            return
 
     if service_utils.is_glance_image(image_href):
         properties = ['kernel_id', 'ramdisk_id']
@@ -589,6 +606,7 @@ def validate_image_properties(task, deploy_info):
             properties.append('stage2_id')
         image_props = get_image_properties(task.context, image_href)
     else:
+        # We are likely netbooting in this case...
         properties = ['kernel', 'ramdisk']
         image_props = {}
 
@@ -613,17 +631,11 @@ def get_boot_option(node):
     :returns: A string representing the boot option type. Defaults to
         configuration setting [deploy]default_boot_mode.
     """
-
-    # NOTE(TheJulia): Software raid always implies local deployment
-    if is_software_raid(node):
-        return 'local'
     if is_anaconda_deploy(node):
         return 'kickstart'
     if is_ramdisk_deploy(node):
         return 'ramdisk'
-    capabilities = utils.parse_instance_info_capabilities(node)
-    return capabilities.get('boot_option',
-                            CONF.deploy.default_boot_option).lower()
+    return 'local'
 
 
 # FIXME(dtantsur): relying on deploy interface name is an anti-pattern.
@@ -772,6 +784,81 @@ def tear_down_inband_cleaning(task, manage_boot=True):
             task, power_state_to_restore)
 
 
+def prepare_inband_service(task):
+    """Boot a service ramdisk on the node.
+
+    :param task: a TaskManager instance.
+    :raises: NetworkError if the tenant ports cannot be removed.
+    :raises: InvalidParameterValue when the wrong power state is specified
+         or the wrong driver info is specified for power management.
+    :raises: other exceptions by the node's power driver if something
+            wrong occurred during the power action.
+    :raises: any boot interface's prepare_ramdisk exceptions.
+    :returns: Returns states.SERVICEWAIT
+    """
+    manager_utils.node_power_action(task, states.POWER_OFF)
+    # NOTE(TheJulia): Revealing that the power is off at any time can
+    # cause external power sync to decide that the node must be off.
+    # This may result in a post-rescued instance being turned off
+    # unexpectedly after rescue has started.
+    # TODO(TheJulia): Once we have power/state callbacks to nova,
+    # the reset of the power_state can be removed.
+    task.node.power_state = states.POWER_ON
+    task.node.save()
+
+    task.driver.boot.clean_up_instance(task)
+    with manager_utils.power_state_for_network_configuration(task):
+        task.driver.network.unconfigure_tenant_networks(task)
+        task.driver.network.add_servicing_network(task)
+    if CONF.agent.manage_agent_boot:
+        # prepare_ramdisk will set the boot device
+        prepare_agent_boot(task)
+    manager_utils.node_power_action(task, states.POWER_ON)
+
+    return states.SERVICEWAIT
+
+
+def tear_down_inband_service(task):
+    """Tears down the environment setup for in-band service.
+
+    This method does the following:
+    1. Powers off the bare metal node (unless the node is fast
+    tracked or there was a service failure).
+    2. If 'manage_boot' parameter is set to true, it also calls
+    the 'clean_up_ramdisk' method of boot interface to clean
+    up the environment that was set for booting agent ramdisk.
+    3. Deletes the cleaning ports which were setup as part
+    of cleaning.
+
+    :param task: a TaskManager object containing the node
+    :raises: NetworkError, NodeServiceFailure if the cleaning ports cannot be
+        removed.
+    """
+    node = task.node
+    service_failure = (node.fault == faults.SERVICE_FAILURE)
+
+    if not service_failure:
+        manager_utils.node_power_action(task, states.POWER_OFF)
+
+    task.driver.boot.clean_up_ramdisk(task)
+
+    power_state_to_restore = manager_utils.power_on_node_if_needed(task)
+
+    if not service_failure:
+        manager_utils.restore_power_state_if_needed(
+            task, power_state_to_restore)
+
+        with manager_utils.power_state_for_network_configuration(task):
+            task.driver.network.remove_servicing_network(task)
+            task.driver.network.configure_tenant_networks(task)
+        task.driver.boot.prepare_instance(task)
+        manager_utils.restore_power_state_if_needed(
+            task, power_state_to_restore)
+
+        # Change the task instead of return the state.
+        task.process_event('done')
+
+
 def get_image_instance_info(node):
     """Gets the image information from the node.
 
@@ -818,7 +905,7 @@ def get_image_instance_info(node):
 
     error_msg = (_("Cannot validate image information for node %s because one "
                    "or more parameters are missing from its instance_info and "
-                   "insufficent information is present to boot from a remote "
+                   "insufficient information is present to boot from a remote "
                    "volume")
                  % node.uuid)
     check_for_missing_params(info, error_msg)
@@ -829,7 +916,7 @@ def get_image_instance_info(node):
 _ERR_MSG_INVALID_DEPLOY = _("Invalid parameter %(param)s: %(reason)s")
 
 
-def parse_instance_info(node):
+def parse_instance_info(node, image_deploy=True):
     """Gets the instance specific Node deployment info.
 
     This method validates whether the 'instance_info' property of the
@@ -837,6 +924,9 @@ def parse_instance_info(node):
     deploy images to the node.
 
     :param node: a single Node.
+    :param image_deploy: If the deployment interface is aware this
+                         is an image based deployment, default
+                         True.
     :returns: A dict with the instance_info values.
     :raises: MissingParameterValue, if any of the required parameters are
         missing.
@@ -856,7 +946,12 @@ def parse_instance_info(node):
                     i_info['image_source'])):
             i_info['kernel'] = info.get('kernel')
             i_info['ramdisk'] = info.get('ramdisk')
-        i_info['root_gb'] = info.get('root_gb')
+        if image_deploy:
+            # root_gb is expected with partition images.
+            i_info['root_gb'] = info.get('root_gb')
+
+    # NOTE(TheJulia): Kernel/ramdisk may be optional and originated
+    # with pure workload network booting.
 
     error_msg = _("Some parameters were missing in node's instance_info")
     check_for_missing_params(i_info, error_msg)
@@ -879,7 +974,8 @@ def parse_instance_info(node):
             err_msg_invalid = _("Cannot deploy whole disk image with "
                                 "swap or ephemeral size set")
             raise exception.InvalidParameterValue(err_msg_invalid)
-    else:
+    elif image_deploy:
+        # NOTE(TheJulia): This only applies to partition image deploys.
         _validate_layout_properties(node, info, i_info)
 
     i_info['configdrive'] = info.get('configdrive')
@@ -990,7 +1086,8 @@ class InstanceImageCache(image_cache.ImageCache):
 
 
 @METRICS.timer('cache_instance_image')
-def cache_instance_image(ctx, node, force_raw=None):
+def cache_instance_image(ctx, node, force_raw=None, expected_format=None,
+                         expected_checksum=None, expected_checksum_algo=None):
     """Fetch the instance's image from Glance
 
     This method pulls the disk image and writes them to the appropriate
@@ -999,8 +1096,16 @@ def cache_instance_image(ctx, node, force_raw=None):
     :param ctx: context
     :param node: an ironic node object
     :param force_raw: whether convert image to raw format
+    :param expected_format: The expected format of the disk image contents.
+    :param expected_checksum: The expected image checksum, to be used if we
+           need to convert the image to raw prior to deploying.
+    :param expected_checksum_algo: The checksum algo in use, if separately
+           set.
     :returns: a tuple containing the uuid of the image and the path in
         the filesystem where image is cached.
+    :raises: InvalidImage if the requested image is invalid and cannot be
+        used for deployed based upon contents of the image or the metadata
+        surrounding the image not matching the configured image.
     """
     # NOTE(dtantsur): applying the default here to make the option mutable
     if force_raw is None:
@@ -1014,10 +1119,11 @@ def cache_instance_image(ctx, node, force_raw=None):
     LOG.debug("Fetching image %(image)s for node %(uuid)s",
               {'image': uuid, 'uuid': node.uuid})
 
-    fetch_images(ctx, InstanceImageCache(), [(uuid, image_path)],
-                 force_raw)
-
-    return (uuid, image_path)
+    image_list = fetch_images(ctx, InstanceImageCache(), [(uuid, image_path)],
+                              force_raw, expected_format=expected_format,
+                              expected_checksum=expected_checksum,
+                              expected_checksum_algo=expected_checksum_algo)
+    return (uuid, image_path, image_list[0][2])
 
 
 @METRICS.timer('destroy_images')
@@ -1034,17 +1140,11 @@ def destroy_images(node_uuid):
 @METRICS.timer('compute_image_checksum')
 def compute_image_checksum(image_path, algorithm='md5'):
     """Compute checksum by given image path and algorithm."""
-    time_start = time.time()
-    LOG.debug('Start computing %(algo)s checksum for image %(image)s.',
-              {'algo': algorithm, 'image': image_path})
-    checksum = fileutils.compute_file_checksum(image_path,
-                                               algorithm=algorithm)
-    time_elapsed = time.time() - time_start
-    LOG.debug('Computed %(algo)s checksum for image %(image)s in '
-              '%(delta).2f seconds, checksum value: %(checksum)s.',
-              {'algo': algorithm, 'image': image_path, 'delta': time_elapsed,
-               'checksum': checksum})
-    return checksum
+    # NOTE(TheJulia): This likely wouldn't be removed, but if we do
+    # significant refactoring we could likely just change everything
+    # over to the images common code, if we don't need the metrics
+    # data anymore.
+    return checksum_utils.compute_image_checksum(image_path, algorithm)
 
 
 def remove_http_instance_symlink(node_uuid):
@@ -1058,28 +1158,102 @@ def destroy_http_instance_images(node):
     destroy_images(node.uuid)
 
 
-def _validate_image_url(node, url, secret=False):
+def _validate_image_url(node, url, secret=False, inspect_image=None,
+                        expected_format=None):
     """Validates image URL through the HEAD request.
 
     :param url: URL to be validated
     :param secret: if URL is secret (e.g. swift temp url),
         it will not be shown in logs.
+    :param inspect_image: If the requested URL should have extensive
+        content checking applied. Defaults to the value provided by
+        the [conductor]conductor_always_validates_images configuration
+        parameter setting, but is also able to be turned off by supplying
+        False where needed to perform a redirect or URL head request only.
+    :param expected_format: The expected image format, if known, for
+        the image inspection logic.
+    :returns: Returns a dictionary with basic information about the
+              requested image if image introspection is
     """
+    if inspect_image is not None:
+        # The caller has a bit more context and we can rely upon it,
+        # for example if it knows we cannot or should not inspect
+        # the image contents.
+        inspect = inspect_image
+    elif not CONF.conductor.disable_deep_image_inspection:
+        inspect = CONF.conductor.conductor_always_validates_images
+    else:
+        # If we're here, file inspection has been explicitly disabled.
+        inspect = False
+
+    # NOTE(TheJulia): This method gets used in two different ways.
+    # The first is as a "i did a thing, let me make sure my url works."
+    # The second is to validate a remote URL is valid. In the remote case
+    # we will grab the file and proceed from there.
+    image_info = {}
     try:
+        # NOTE(TheJulia): This method only validates that an exception
+        # is NOT raised. In other words, that the endpoint does not
+        # return a 200. If we're fed a folder list, this will still
+        # work, which is a good and bad thing at the same time. :/
         image_service.HttpImageService().validate_href(url, secret)
     except exception.ImageRefValidationFailed as e:
         with excutils.save_and_reraise_exception():
             LOG.error("The specified URL is not a valid HTTP(S) URL or is "
                       "not reachable for node %(node)s: %(msg)s",
                       {'node': node.uuid, 'msg': e})
+    if inspect:
+        LOG.info("Inspecting image contents for %(node)s with url %(url)s. "
+                 "Expecting user supplied format: %(expected)s",
+                 {'node': node.uuid,
+                  'expected': expected_format,
+                  'url': url})
+        # Utilizes the file cache since it knows how to pull files down
+        # and handles pathing and caching and all that fun, however with
+        # force_raw set as false.
+
+        # The goal here being to get the file we would normally just point
+        # IPA at, be it via swift transfer *or* direct URL request, and
+        # perform the safety check on it before allowing it to proceed.
+        ctx = context.get_admin_context()
+        # NOTE(TheJulia): Because we're using the image cache here, we
+        # let it run the image validation checking as it's normal course
+        # of action, and save what it tells us the image format is.
+        # if there *was* a mismatch, it will raise the error.
+
+        # NOTE(TheJulia): We don't need to supply the checksum here, because
+        # we are not converting the image. The net result is the deploy
+        # interface or remote agent has the responsibility to checksum the
+        # image.
+        _, image_path, img_format = cache_instance_image(
+            ctx,
+            node,
+            force_raw=False,
+            expected_format=expected_format)
+        # NOTE(TheJulia): We explicitly delete this file because it has no use
+        # in the cache after this point.
+        il_utils.unlink_without_raise(image_path)
+        image_info['disk_format'] = img_format
+    return image_info
 
 
 def _cache_and_convert_image(task, instance_info, image_info=None):
     """Cache an image locally and covert it to RAW if needed."""
     # Ironic cache and serve images from httpboot server
     force_raw = direct_deploy_should_convert_raw_image(task.node)
-    _, image_path = cache_instance_image(task.context, task.node,
-                                         force_raw=force_raw)
+
+    if image_info is None:
+        initial_format = instance_info.get('image_disk_format')
+    else:
+        initial_format = image_info.get('disk_format')
+    checksum, checksum_algo = checksum_utils.get_checksum_and_algo(
+        instance_info)
+    _, image_path, img_format = cache_instance_image(
+        task.context, task.node,
+        force_raw=force_raw,
+        expected_format=initial_format,
+        expected_checksum=checksum,
+        expected_checksum_algo=checksum_algo)
     if force_raw or image_info is None:
         if force_raw:
             instance_info['image_disk_format'] = 'raw'
@@ -1096,21 +1270,30 @@ def _cache_and_convert_image(task, instance_info, image_info=None):
         # sha256.
         if image_info is None:
             os_hash_algo = instance_info.get('image_os_hash_algo')
+            hash_value = instance_info.get('image_os_hash_value')
+            old_checksum = instance_info.get('image_checksum')
         else:
             os_hash_algo = image_info.get('os_hash_algo')
+            hash_value = image_info.get('os_hash_value')
+            old_checksum = image_info.get('checksum')
 
-        if not os_hash_algo or os_hash_algo == 'md5':
-            LOG.debug("Checksum algorithm for image %(image)s for node "
-                      "%(node)s is set to '%(algo)s', changing to 'sha256'",
-                      {'algo': os_hash_algo, 'node': task.node.uuid,
-                       'image': image_path})
-            os_hash_algo = 'sha256'
+        if initial_format != instance_info['image_disk_format']:
+            if not os_hash_algo or os_hash_algo == 'md5':
+                LOG.debug("Checksum algorithm for image %(image)s for node "
+                          "%(node)s is set to '%(algo)s', changing to sha256",
+                          {'algo': os_hash_algo, 'node': task.node.uuid,
+                           'image': image_path})
+                os_hash_algo = 'sha256'
+
+            LOG.debug('Recalculating checksum for image %(image)s for node '
+                      '%(node)s due to image conversion',
+                      {'image': image_path, 'node': task.node.uuid})
+            instance_info['image_checksum'] = None
+            hash_value = checksum_utils.compute_image_checksum(image_path,
+                                                               os_hash_algo)
+        else:
+            instance_info['image_checksum'] = old_checksum
 
-        LOG.debug('Recalculating checksum for image %(image)s for node '
-                  '%(node)s due to image conversion',
-                  {'image': image_path, 'node': task.node.uuid})
-        instance_info['image_checksum'] = None
-        hash_value = compute_image_checksum(image_path, os_hash_algo)
         instance_info['image_os_hash_algo'] = os_hash_algo
         instance_info['image_os_hash_value'] = hash_value
     else:
@@ -1153,7 +1336,11 @@ def _cache_and_convert_image(task, instance_info, image_info=None):
          task.node.uuid])
     if file_extension:
         http_image_url = http_image_url + file_extension
-    _validate_image_url(task.node, http_image_url, secret=False)
+    # We don't inspect the image in our url check because we just need to do
+    # an quick path validity check here, we should be checking contents way
+    # earlier on in this method.
+    _validate_image_url(task.node, http_image_url, secret=False,
+                        inspect_image=False)
     instance_info['image_url'] = http_image_url
 
 
@@ -1178,25 +1365,57 @@ def build_instance_info_for_deploy(task):
     instance_info = node.instance_info
     iwdi = node.driver_internal_info.get('is_whole_disk_image')
     image_source = instance_info['image_source']
+
+    # Flag if we know the source is a path, used for Anaconda
+    # deploy interface where you can just tell anaconda to
+    # consume artifacts from a path. In this case, we are not
+    # doing any image conversions, we're just passing through
+    # a URL in the form of configuration.
+    isap = node.driver_internal_info.get('is_source_a_path')
+
+    # If our url ends with a /, i.e. we have been supplied with a path,
+    # we can only deploy this in limited cases for drivers and tools
+    # which are aware of such. i.e. anaconda.
     image_download_source = get_image_download_source(node)
     boot_option = get_boot_option(task.node)
 
+    # There is no valid reason this should already be set, and
+    # and gets replaced at various points in this sequence.
+    instance_info['image_url'] = None
+
     if service_utils.is_glance_image(image_source):
         glance = image_service.GlanceImageService(context=task.context)
         image_info = glance.show(image_source)
         LOG.debug('Got image info: %(info)s for node %(node)s.',
                   {'info': image_info, 'node': node.uuid})
+        # Values are explicitly set into the instance info field
+        # so IPA have the values available.
+        instance_info['image_checksum'] = image_info['checksum']
+        instance_info['image_os_hash_algo'] = image_info['os_hash_algo']
+        instance_info['image_os_hash_value'] = image_info['os_hash_value']
         if image_download_source == 'swift':
+            # In this case, we are getting a file *from* swift for a glance
+            # image which is backed by swift. IPA downloads the file directly
+            # from swift, but cannot get any metadata related to it otherwise.
             swift_temp_url = glance.swift_temp_url(image_info)
-            _validate_image_url(node, swift_temp_url, secret=True)
+            image_format = image_info.get('disk_format')
+            # In the process of validating the URL is valid, we will perform
+            # the requisite safety checking of the asset as we can end up
+            # converting it in the agent, or needing the disk format value
+            # to be correct for the Ansible deployment interface.
+            validate_results = _validate_image_url(
+                node, swift_temp_url, secret=True,
+                expected_format=image_format)
             instance_info['image_url'] = swift_temp_url
-            instance_info['image_checksum'] = image_info['checksum']
-            instance_info['image_disk_format'] = image_info['disk_format']
-            instance_info['image_os_hash_algo'] = image_info['os_hash_algo']
-            instance_info['image_os_hash_value'] = image_info['os_hash_value']
+            instance_info['image_disk_format'] = \
+                validate_results.get('disk_format', image_format)
         else:
+            # In this case, we're directly downloading the glance image and
+            # hosting it locally for retrieval by the IPA.
             _cache_and_convert_image(task, instance_info, image_info)
 
+        # We're just populating extra information for a glance backed image in
+        # case a deployment interface driver needs them at some point.
         instance_info['image_container_format'] = (
             image_info['container_format'])
         instance_info['image_tags'] = image_info.get('tags', [])
@@ -1207,17 +1426,94 @@ def build_instance_info_for_deploy(task):
             instance_info['ramdisk'] = image_info['properties']['ramdisk_id']
     elif (image_source.startswith('file://')
           or image_download_source == 'local'):
+        # In this case, we're explicitly downloading (or copying a file)
+        # hosted locally so IPA can download it directly from Ironic.
+
+        # NOTE(TheJulia): Intentionally only supporting file:/// as image
+        # based deploy source since we don't want to, nor should we be in
+        # in the business of copying large numbers of files as it is a
+        # huge performance impact.
+
         _cache_and_convert_image(task, instance_info)
     else:
-        _validate_image_url(node, image_source)
-        instance_info['image_url'] = image_source
+        # This is the "all other cases" logic for aspects like the user
+        # has supplied us a direct URL to reference. In cases like the
+        # anaconda deployment interface where we might just have a path
+        # and not a file, or where a user may be supplying a full URL to
+        # a remotely hosted image, we at a minimum need to check if the url
+        # is valid, and address any redirects upfront.
+        try:
+            # NOTE(TheJulia): In the case we're here, we not doing an
+            # integrated image based deploy, but we may also be doing
+            # a path based anaconda base deploy, in which case we have
+            # no backing image, but we need to check for a URL
+            # redirection. So, if the source is a path (i.e. isap),
+            # we don't need to inspect the image as there is no image
+            # in the case for the deployment to drive.
+            validated_results = {}
+            if isap:
+                # This is if the source is a path url, such as one used by
+                # anaconda templates to to rely upon bootstrapping defaults.
+                _validate_image_url(node, image_source, inspect_image=False)
+            else:
+                # When not isap, we can just let _validate_image_url make a
+                # the required decision on if contents need to be sampled,
+                # or not. We try to pass the image_disk_format which may be
+                # declared by the user, and if not we set expected_format to
+                # None.
+                validate_results = _validate_image_url(
+                    node,
+                    image_source,
+                    expected_format=instance_info.get('image_disk_format',
+                                                      None))
+            # image_url is internal, and used by IPA and some boot templates.
+            # in most cases, it needs to come from image_source explicitly.
+            if 'disk_format' in validated_results:
+                # Ensure IPA has the value available, so write what we detect,
+                # if anything. This is also an item which might be needful
+                # with ansible deploy interface, when used in standalone mode.
+                instance_info['image_disk_format'] = \
+                    validate_results.get('disk_format')
+            instance_info['image_url'] = image_source
+        except exception.ImageRefIsARedirect as e:
+            # At this point, we've got a redirect response from the webserver,
+            # and we're going to try to handle it as a single redirect action,
+            # as requests, by default, only lets a single redirect to occur.
+            # This is likely a URL pathing fix, like a trailing / on a path,
+            # or move to HTTPS from a user supplied HTTP url.
+            if e.redirect_url:
+                # Since we've got a redirect, we need to carry the rest of the
+                # request logic as well, which includes recording a disk
+                # format, if applicable.
+                instance_info['image_url'] = e.redirect_url
+                # We need to save the image_source back out so it caches
+                instance_info['image_source'] = e.redirect_url
+                task.node.instance_info = instance_info
+                if not isap:
+                    # The redirect doesn't relate to a path being used, so
+                    # the target is a filename, likely cause is webserver
+                    # telling the client to use HTTPS.
+                    validated_results = _validate_image_url(
+                        node, e.redirect_url,
+                        expected_format=instance_info.get('image_disk_format',
+                                                          None))
+                    if 'disk_format' in validated_results:
+                        instance_info['image_disk_format'] = \
+                            validated_results.get('disk_format')
+            else:
+                raise
 
-    if not iwdi:
-        instance_info['image_type'] = images.IMAGE_TYPE_PARTITION
-        i_info = parse_instance_info(node)
-        instance_info.update(i_info)
+    if not isap:
+        if not iwdi:
+            instance_info['image_type'] = images.IMAGE_TYPE_PARTITION
+            i_info = parse_instance_info(node)
+            instance_info.update(i_info)
+        else:
+            instance_info['image_type'] = images.IMAGE_TYPE_WHOLE_DISK
     else:
-        instance_info['image_type'] = images.IMAGE_TYPE_WHOLE_DISK
+        # Call central parsing so we retain things like config drives.
+        i_info = parse_instance_info(node, image_deploy=False)
+        instance_info.update(i_info)
     return instance_info
 
 
@@ -1330,7 +1626,6 @@ def is_iscsi_boot(task):
 
 # NOTE(etingof): retain original location of these funcs for compatibility
 is_secure_boot_requested = boot_mode_utils.is_secure_boot_requested
-is_trusted_boot_requested = boot_mode_utils.is_trusted_boot_requested
 get_boot_mode_for_deploy = boot_mode_utils.get_boot_mode_for_deploy
 parse_instance_info_capabilities = (
     utils.parse_instance_info_capabilities
@@ -1415,10 +1710,12 @@ def prepare_agent_boot(task):
     task.driver.boot.prepare_ramdisk(task, deploy_opts)
 
 
-def reboot_to_finish_step(task):
+def reboot_to_finish_step(task, timeout=None):
     """Reboot the node into IPA to finish a deploy/clean step.
 
     :param task: a TaskManager instance.
+    :param timeout: timeout (in seconds) positive integer (> 0) for any
+      power state. ``None`` indicates to use default timeout.
     :returns: states.CLEANWAIT if cleaning operation in progress
               or states.DEPLOYWAIT if deploy operation in progress.
     """
@@ -1431,7 +1728,7 @@ def reboot_to_finish_step(task):
             manager_utils.node_power_action(task, states.POWER_OFF)
         prepare_agent_boot(task)
 
-    manager_utils.node_power_action(task, states.REBOOT)
+    manager_utils.node_power_action(task, states.REBOOT, timeout)
     return get_async_step_return_state(task.node)
 
 
@@ -1470,4 +1767,4 @@ def needs_agent_ramdisk(node, mode='deploy'):
     # Ramdisk deploys don't need an agent, but cleaning will. Since we don't
     # want nodes to be stuck on deletion, require an agent when cleaning is
     # enabled.
-    return not manager_utils.skip_automated_cleaning(node, log=False)
+    return not manager_utils.skip_automated_cleaning(node)
