@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import itertools
 import math
 
 from ironic_lib import metrics_utils
@@ -22,6 +23,7 @@ from oslo_utils import units
 
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import raid as raid_common
 from ironic.common import raid
 from ironic.common import states
 from ironic.conductor import periodics
@@ -41,7 +43,6 @@ RAID_LEVELS = {
         'min_disks': 1,
         'max_disks': 1000,
         'type': 'simple',
-        'volume_type': 'NonRedundant',
         'raid_type': 'RAID0',
         'overhead': 0
     },
@@ -49,7 +50,6 @@ RAID_LEVELS = {
         'min_disks': 2,
         'max_disks': 2,
         'type': 'simple',
-        'volume_type': 'Mirrored',
         'raid_type': 'RAID1',
         'overhead': 1
     },
@@ -57,7 +57,6 @@ RAID_LEVELS = {
         'min_disks': 3,
         'max_disks': 1000,
         'type': 'simple',
-        'volume_type': 'StripedWithParity',
         'raid_type': 'RAID5',
         'overhead': 1
     },
@@ -65,25 +64,21 @@ RAID_LEVELS = {
         'min_disks': 4,
         'max_disks': 1000,
         'type': 'simple',
-        'volume_type': 'StripedWithParity',
         'raid_type': 'RAID6',
         'overhead': 2
     },
     '1+0': {
         'type': 'spanned',
-        'volume_type': 'SpannedMirrors',
         'raid_type': 'RAID10',
         'span_type': '1'
     },
     '5+0': {
         'type': 'spanned',
-        'volume_type': 'SpannedStripesWithParity',
         'raid_type': 'RAID50',
         'span_type': '5'
     },
     '6+0': {
         'type': 'spanned',
-        'volume_type': 'SpannedStripesWithParity',
         'raid_type': 'RAID60',
         'span_type': '6'
     }
@@ -616,17 +611,19 @@ def _drive_path(storage, drive_id):
 def _construct_volume_payload(
         node, storage, raid_controller, physical_disks, raid_level, size_bytes,
         disk_name=None, span_length=None, span_depth=None):
-    payload = {'Encrypted': False,
-               'VolumeType': RAID_LEVELS[raid_level]['volume_type'],
-               'RAIDType': RAID_LEVELS[raid_level]['raid_type'],
-               'CapacityBytes': size_bytes}
+    payload = {
+        'RAIDType': RAID_LEVELS[raid_level]['raid_type'],
+        'CapacityBytes': size_bytes
+    }
     if physical_disks:
         payload['Links'] = {
             "Drives": [{"@odata.id": _drive_path(storage, d)} for d in
                        physical_disks]
         }
+    if disk_name:
+        payload['Name'] = disk_name
     LOG.debug('Payload for RAID logical disk creation on node %(node_uuid)s: '
-              '%(payload)r', {'node': node.uuid, 'payload': payload})
+              '%(payload)r', {'node_uuid': node.uuid, 'payload': payload})
     return payload
 
 
@@ -684,6 +681,38 @@ def create_virtual_disk(task, raid_controller, physical_disks, raid_level,
         raise exception.RedfishError(error=exc)
 
 
+def update_raid_config(node):
+    """Updates node's raid_config field with current logical disks.
+
+    :param node: node for which to update the raid_config field
+    """
+    system = redfish_utils.get_system(node)
+    logical_disks = []
+    vol_no_raid_type = []
+    for stor in system.storage.get_members():
+        for vol in stor.volumes.get_members():
+            if vol.raid_type:
+                logical_disk = {
+                    'id': vol.identity,
+                    'name': vol.name,
+                    'controller': stor.identity,
+                    'size_gb': int(vol.capacity_bytes / units.Gi),
+                    'raid_level': next(
+                        key for key, value in RAID_LEVELS.items()
+                        if value['raid_type'] == vol.raid_type.value)
+                }
+                logical_disks.append(logical_disk)
+            else:
+                vol_no_raid_type.append(vol.identity)
+
+    if vol_no_raid_type:
+        LOG.warning("Unable to update raid_config for volumes missing RAID "
+                    "type: %(vol_no_raid_type)s",
+                    {'vol_no_raid_type': ", ".join(vol_no_raid_type)})
+
+    raid_common.update_raid_info(node, {'logical_disks': logical_disks})
+
+
 class RedfishRAID(base.RAIDInterface):
 
     def __init__(self):
@@ -700,32 +729,6 @@ class RedfishRAID(base.RAIDInterface):
         """
         return redfish_utils.COMMON_PROPERTIES.copy()
 
-    def _validate_vendor(self, task):
-        vendor = task.node.properties.get('vendor')
-        if not vendor:
-            return
-
-        if 'dell' in vendor.lower().split():
-            raise exception.InvalidParameterValue(
-                _("The %(iface)s raid interface is not suitable for node "
-                  "%(node)s with vendor %(vendor)s, use idrac-redfish instead")
-                % {'iface': task.node.get_interface('raid'),
-                   'node': task.node.uuid, 'vendor': vendor})
-
-    def validate(self, task):
-        """Validates the RAID Interface.
-
-        This method validates the properties defined by Ironic for RAID
-        configuration. Driver implementations of this interface can override
-        this method for doing more validations (such as BMC's credentials).
-
-        :param task: A TaskManager instance.
-        :raises: InvalidParameterValue, if the RAID configuration is invalid.
-        :raises: MissingParameterValue, if some parameters are missing.
-        """
-        self._validate_vendor(task)
-        super(RedfishRAID, self).validate(task)
-
     def validate_raid_config(self, task, raid_config):
         """Validates the given RAID configuration.
 
@@ -744,6 +747,8 @@ class RedfishRAID(base.RAIDInterface):
             raise exception.InvalidParameterValue(
                 _('interface type `scsi` not supported by Redfish RAID'))
 
+    @base.service_step(priority=0,
+                       argsinfo=base.RAID_APPLY_CONFIGURATION_ARGSINFO)
     @base.deploy_step(priority=0,
                       argsinfo=base.RAID_APPLY_CONFIGURATION_ARGSINFO)
     def apply_configuration(self, task, raid_config, create_root_volume=True,
@@ -817,31 +822,14 @@ class RedfishRAID(base.RAIDInterface):
         logical_disks_to_create = self.pre_create_configuration(
             task, logical_disks_to_create)
 
-        reboot_required = False
-        raid_configs = list()
-        for logical_disk in logical_disks_to_create:
-            raid_config = dict()
-            response = create_virtual_disk(
-                task,
-                raid_controller=logical_disk.get('controller'),
-                physical_disks=logical_disk['physical_disks'],
-                raid_level=logical_disk['raid_level'],
-                size_bytes=logical_disk['size_bytes'],
-                disk_name=logical_disk.get('name'),
-                span_length=logical_disk.get('span_length'),
-                span_depth=logical_disk.get('span_depth'),
-                error_handler=self.volume_create_error_handler)
-            # only save the async tasks (task_monitors) in raid_config
-            if (response is not None
-                    and hasattr(response, 'task_monitor_uri')):
-                raid_config['operation'] = 'create'
-                raid_config['raid_controller'] = logical_disk.get(
-                    'controller')
-                raid_config['task_monitor_uri'] = response.task_monitor_uri
-                reboot_required = True
-                raid_configs.append(raid_config)
-
-        node.set_driver_internal_info('raid_configs', raid_configs)
+        # Group logical disks by controller
+        def gb_key(x):
+            return x.get('controller')
+        gb_list = itertools.groupby(
+            sorted(logical_disks_to_create, key=gb_key), gb_key)
+        ld_grouped = {k: list(g) for k, g in gb_list}
+        raid_configs, reboot_required = self._submit_create_configuration(
+            task, ld_grouped)
 
         return_state = None
         deploy_utils.set_async_step_flags(
@@ -851,12 +839,15 @@ class RedfishRAID(base.RAIDInterface):
             polling=True)
         if reboot_required:
             return_state = deploy_utils.reboot_to_finish_step(task)
+        else:
+            update_raid_config(node)
 
         return self.post_create_configuration(
             task, raid_configs, return_state=return_state)
 
     @base.clean_step(priority=0)
     @base.deploy_step(priority=0)
+    @base.service_step(priority=0)
     def delete_configuration(self, task):
         """Delete RAID configuration on the node.
 
@@ -866,53 +857,8 @@ class RedfishRAID(base.RAIDInterface):
             complete.
         """
         node = task.node
-        system = redfish_utils.get_system(node)
-        vols_to_delete = []
-        try:
-            for storage in system.storage.get_members():
-                controller = (storage.storage_controllers[0]
-                              if storage.storage_controllers else None)
-                controller_id = None
-                if controller:
-                    controller_id = storage.identity
-                for volume in storage.volumes.get_members():
-                    if (volume.raid_type or volume.volume_type not in
-                            [None, sushy.VOLUME_TYPE_RAW_DEVICE]):
-                        vols_to_delete.append((storage.volumes, volume,
-                                               controller_id))
-        except sushy.exceptions.SushyError as exc:
-            error_msg = _('Cannot get the list of volumes to delete for node '
-                          '%(node_uuid)s. Reason: %(error)s.' %
-                          {'node_uuid': node.uuid, 'error': exc})
-            LOG.error(error_msg)
-            raise exception.RedfishError(error=exc)
-
-        self.pre_delete_configuration(task, vols_to_delete)
-
-        reboot_required = False
-        raid_configs = list()
-        for vol_coll, volume, controller_id in vols_to_delete:
-            raid_config = dict()
-            apply_time = None
-            apply_time_support = vol_coll.operation_apply_time_support
-            if (apply_time_support
-                    and apply_time_support.mapped_supported_values):
-                supported_values = apply_time_support.mapped_supported_values
-                if sushy.APPLY_TIME_IMMEDIATE in supported_values:
-                    apply_time = sushy.APPLY_TIME_IMMEDIATE
-                elif sushy.APPLY_TIME_ON_RESET in supported_values:
-                    apply_time = sushy.APPLY_TIME_ON_RESET
-            response = volume.delete(apply_time=apply_time)
-            # only save the async tasks (task_monitors) in raid_config
-            if (response is not None
-                    and hasattr(response, 'task_monitor_uri')):
-                raid_config['operation'] = 'delete'
-                raid_config['raid_controller'] = controller_id
-                raid_config['task_monitor_uri'] = response.task_monitor_uri
-                reboot_required = True
-                raid_configs.append(raid_config)
-
-        node.set_driver_internal_info('raid_configs', raid_configs)
+        raid_configs, reboot_required = self._submit_delete_configuration(
+            task)
 
         return_state = None
         deploy_utils.set_async_step_flags(
@@ -922,6 +868,8 @@ class RedfishRAID(base.RAIDInterface):
             polling=True)
         if reboot_required:
             return_state = deploy_utils.reboot_to_finish_step(task)
+        else:
+            update_raid_config(node)
 
         return self.post_delete_configuration(
             task, raid_configs, return_state=return_state)
@@ -1027,6 +975,7 @@ class RedfishRAID(base.RAIDInterface):
 
         task.upgrade_lock()
         self._clear_raid_configs(task.node)
+        update_raid_config(task.node)
 
     @METRICS.timer('RedfishRAID._query_raid_config_status')
     @periodics.node_periodic(
@@ -1041,14 +990,15 @@ class RedfishRAID(base.RAIDInterface):
         """Periodic job to check RAID config tasks."""
         self._check_node_raid_config(task)
 
-    def _raid_config_in_progress(self, task, raid_config):
+    def _raid_config_in_progress(self, task, task_monitor_uri, operation):
         """Check if this RAID configuration operation is still in progress.
 
         :param task: TaskManager object containing the node.
-        :param raid_config: RAID configuration operation details.
+        :param task_monitor_uri: Redfish task monitor URI
+        :param operation: 'create' or 'delete' operation for given task.
+            Used in log messages.
         :returns: True, if still in progress, otherwise False.
         """
-        task_monitor_uri = raid_config['task_monitor_uri']
         try:
             task_monitor = redfish_utils.get_task_monitor(task.node,
                                                           task_monitor_uri)
@@ -1056,14 +1006,14 @@ class RedfishRAID(base.RAIDInterface):
             LOG.info('Unable to get status of RAID %(operation)s task '
                      '%(task_mon_uri)s to node %(node_uuid)s; assuming task '
                      'completed successfully',
-                     {'operation': raid_config['operation'],
+                     {'operation': operation,
                       'task_mon_uri': task_monitor_uri,
                       'node_uuid': task.node.uuid})
             return False
         if task_monitor.is_processing:
             LOG.debug('RAID %(operation)s task %(task_mon_uri)s to node '
                       '%(node_uuid)s still in progress',
-                      {'operation': raid_config['operation'],
+                      {'operation': operation,
                        'task_mon_uri': task_monitor.task_monitor_uri,
                        'node_uuid': task.node.uuid})
             return True
@@ -1080,13 +1030,13 @@ class RedfishRAID(base.RAIDInterface):
                     [sushy.HEALTH_OK, sushy.HEALTH_WARNING]):
                 LOG.info('RAID %(operation)s task %(task_mon_uri)s to node '
                          '%(node_uuid)s completed.',
-                         {'operation': raid_config['operation'],
+                         {'operation': operation,
                           'task_mon_uri': task_monitor.task_monitor_uri,
                           'node_uuid': task.node.uuid})
             else:
                 LOG.error('RAID %(operation)s task %(task_mon_uri)s to node '
                           '%(node_uuid)s failed; messages: %(messages)s',
-                          {'operation': raid_config['operation'],
+                          {'operation': operation,
                            'task_mon_uri': task_monitor.task_monitor_uri,
                            'node_uuid': task.node.uuid,
                            'messages': ", ".join(messages)})
@@ -1094,19 +1044,161 @@ class RedfishRAID(base.RAIDInterface):
 
     @METRICS.timer('RedfishRAID._check_node_raid_config')
     def _check_node_raid_config(self, task):
-        """Check the progress of running RAID config on a node."""
+        """Check the progress of running RAID config on a node.
+
+        :param task: TaskManager object containing the node.
+        """
         node = task.node
         raid_configs = node.driver_internal_info['raid_configs']
 
         task.upgrade_lock()
-        raid_configs[:] = [i for i in raid_configs
-                           if self._raid_config_in_progress(task, i)]
+        raid_configs['task_monitor_uri'] =\
+            [i for i in raid_configs.get('task_monitor_uri')
+             if self._raid_config_in_progress(
+                task, i, raid_configs.get('operation'))]
+        node.set_driver_internal_info('raid_configs', raid_configs)
 
-        if not raid_configs:
-            self._clear_raid_configs(node)
-            LOG.info('RAID configuration completed for node %(node)s',
-                     {'node': node.uuid})
-            if task.node.clean_step:
-                manager_utils.notify_conductor_resume_clean(task)
+        if not raid_configs['task_monitor_uri']:
+            if raid_configs.get('pending'):
+                if raid_configs.get('operation') == 'create':
+                    reboot_required = self._submit_create_configuration(
+                        task, raid_configs.get('pending'))[1]
+                else:
+                    reboot_required = self._submit_delete_configuration(
+                        task)[1]
+                if reboot_required:
+                    deploy_utils.reboot_to_finish_step(task)
             else:
-                manager_utils.notify_conductor_resume_deploy(task)
+                self._clear_raid_configs(node)
+                LOG.info('RAID configuration completed for node %(node)s',
+                         {'node': node.uuid})
+                update_raid_config(task.node)
+                if task.node.clean_step:
+                    manager_utils.notify_conductor_resume_clean(task)
+                else:
+                    manager_utils.notify_conductor_resume_deploy(task)
+
+    def _submit_create_configuration(self, task, ld_grouped):
+        """Processes and submits requests for creating RAID configuration.
+
+        :param task: TaskManager object containing the node.
+        :param ld_grouped: Dictionary of logical disks, grouped by controller.
+
+        :returns: tuple of 1) dictionary containing operation name (create),
+            pending items, and task monitor URIs, and 2) flag indicating if
+            reboot is required.
+        """
+        node = task.node
+        reboot_required = False
+        raid_configs = {'operation': 'create', 'pending': {}}
+        for controller, logical_disks in ld_grouped.items():
+            iter_logical_disks = iter(logical_disks)
+            for logical_disk in iter_logical_disks:
+                response = create_virtual_disk(
+                    task,
+                    raid_controller=logical_disk.get('controller'),
+                    physical_disks=logical_disk['physical_disks'],
+                    raid_level=logical_disk['raid_level'],
+                    size_bytes=logical_disk['size_bytes'],
+                    disk_name=logical_disk.get('volume_name'),
+                    span_length=logical_disk.get('span_length'),
+                    span_depth=logical_disk.get('span_depth'),
+                    error_handler=self.volume_create_error_handler)
+                if (response is not None
+                        and hasattr(response, 'task_monitor_uri')):
+                    raid_configs.setdefault('task_monitor_uri', []).append(
+                        response.task_monitor_uri)
+                    reboot_required = True
+                    # Don't process any on this controller until these
+                    # created to avoid failures where only 1 request
+                    # per controller can be submitted for non-immediate
+                    break
+            # Append remaining disks for this controller, if any left
+            for logical_disk in iter_logical_disks:
+                raid_configs['pending'].setdefault(controller, []).append(
+                    logical_disk)
+
+        # Store only when async operation
+        if reboot_required:
+            node.set_driver_internal_info('raid_configs', raid_configs)
+
+        return raid_configs, reboot_required
+
+    def _submit_delete_configuration(self, task):
+        """Processes and submits requests for deleting virtual disks.
+
+        :param task: TaskManager object containing the node.
+
+        :returns: tuple of 1) dictionary containing operation name (delete),
+            flag to indicate if any disks remaining, and task monitor URIs,
+            and 2) flag indicating if reboot is required
+        :raises RedfishError: if fails to get list of virtual disks
+        """
+        node = task.node
+        system = redfish_utils.get_system(node)
+        vols_to_delete = {}
+        any_left = False
+        try:
+            for storage in system.storage.get_members():
+                controller = (storage.storage_controllers[0]
+                              if storage.storage_controllers else None)
+                controller_id = None
+                if controller:
+                    controller_id = storage.identity
+                iter_volumes = iter(storage.volumes.get_members())
+                for volume in iter_volumes:
+                    if volume.raid_type:
+                        if controller_id not in vols_to_delete:
+                            vols_to_delete[controller_id] = []
+                        apply_time = self._get_apply_time(
+                            storage.volumes.operation_apply_time_support)
+                        vols_to_delete[controller_id].append((
+                            apply_time, volume))
+                        if apply_time == sushy.APPLY_TIME_ON_RESET:
+                            # Don't process any on this controller until these
+                            # deleted to avoid failures where only 1 request
+                            # per controller can be submitted for non-immediate
+                            break
+                any_left = any(iter_volumes)
+        except sushy.exceptions.SushyError as exc:
+            error_msg = _('Cannot get the list of volumes to delete for node '
+                          '%(node_uuid)s. Reason: %(error)s.' %
+                          {'node_uuid': node.uuid, 'error': exc})
+            LOG.error(error_msg)
+            raise exception.RedfishError(error=exc)
+
+        self.pre_delete_configuration(task, vols_to_delete)
+
+        reboot_required = False
+        raid_configs = {'operation': 'delete', 'pending': any_left}
+        for controller, vols_to_delete in vols_to_delete.items():
+            for apply_time, volume in vols_to_delete:
+                response = volume.delete(apply_time=apply_time)
+                # only save the async tasks (task_monitors) in raid_config
+                if (response is not None
+                        and hasattr(response, 'task_monitor_uri')):
+                    raid_configs.setdefault('task_monitor_uri', []).append(
+                        response.task_monitor_uri)
+                    reboot_required = True
+
+        # Store only when async operation
+        if reboot_required:
+            node.set_driver_internal_info('raid_configs', raid_configs)
+
+        return raid_configs, reboot_required
+
+    def _get_apply_time(self, apply_time_support):
+        """Gets apply time for RAID operations
+
+        :param apply_time_support: Supported apply times
+        :returns: None, if supported apply times not specified. Otherwise
+            Immediate when available, or OnReset that will require rebooting.
+        """
+        apply_time = None
+        if apply_time_support and apply_time_support.mapped_supported_values:
+            supported_values = apply_time_support.mapped_supported_values
+            if sushy.APPLY_TIME_IMMEDIATE in supported_values:
+                apply_time = sushy.APPLY_TIME_IMMEDIATE
+            elif sushy.APPLY_TIME_ON_RESET in supported_values:
+                apply_time = sushy.APPLY_TIME_ON_RESET
+        return apply_time
