@@ -47,16 +47,18 @@ def _mock_drive(identity, block_size_bytes=None, capacity_bytes=None,
     )
 
 
-def _mock_volume(identity, volume_type=None, raid_type=None,
-                 capacity_bytes=units.Gi):
+def _mock_volume(identity, raid_type=None,
+                 capacity_bytes=units.Gi, volume_name=None):
     volume = mock.MagicMock(
         _path='/redfish/v1/Systems/1/Storage/1/Volumes/' + identity,
         identity=identity,
-        volume_type=volume_type,
         raid_type=raid_type,
         capacity_bytes=capacity_bytes
     )
-    volume.name = 'Volume ' + identity
+    if volume_name:
+        volume.name = volume_name
+    else:
+        volume.name = 'Volume ' + identity
     # Mocking Immediate that does not return anything
     volume.delete.return_value = None
     return volume
@@ -251,8 +253,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -289,13 +289,14 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 {
                     'size_gb': 100,
                     'raid_level': '5',
-                    'is_root_volume': True
+                    'is_root_volume': True,
+                    'volume_name': 'test-volume'
                 }
             ]
         }
-        created_volumes = [_mock_volume(
-            '1', raid_type=sushy.RAIDType.RAID5,
-            capacity_bytes=100 * units.Gi)]
+        created_volumes = [_mock_volume('1', raid_type=sushy.RAIDType.RAID5,
+                                        capacity_bytes=100 * units.Gi,
+                                        volume_name='test-volume')]
         volumes = mock.MagicMock()
         # Called after volumes created
         volumes.get_members.return_value = created_volumes
@@ -316,8 +317,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -326,7 +325,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                         {'@odata.id': pre + self.drive_id2},
                         {'@odata.id': pre + self.drive_id3}
                     ]
-                }
+                },
+                'Name': 'test-volume'
             }
             self.mock_storage.volumes.create.assert_called_once_with(
                 expected_payload, apply_time=sushy.APPLY_TIME_IMMEDIATE)
@@ -341,7 +341,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             self.assertEqual(
                 [{'controller': 'RAID controller 1',
                   'id': '1',
-                  'name': 'Volume 1',
+                  'name': 'test-volume',
                   'raid_level': '5',
                   'size_gb': 100}],
                 task.node.raid_config['logical_disks'])
@@ -388,8 +388,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -406,7 +404,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 task.node, reboot=True, skip_current_step=True, polling=True)
             mock_get_async_step_return_state.assert_called_once_with(
                 task.node)
-            mock_node_power_action.assert_called_once_with(task, states.REBOOT)
+            mock_node_power_action.assert_called_once_with(
+                task, states.REBOOT, None)
             mock_build_agent_options.assert_called_once_with(task.node)
             self.assertEqual(mock_prepare_ramdisk.call_count, 1)
             # Async operation, raid_config shouldn't be updated yet
@@ -462,8 +461,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload1 = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -475,8 +472,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 }
             }
             expected_payload2 = {
-                'Encrypted': False,
-                'VolumeType': 'Mirrored',
                 'RAIDType': 'RAID1',
                 'CapacityBytes': 536870912000,
                 'Links': {
@@ -556,8 +551,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload = {
-                'Encrypted': False,
-                'VolumeType': 'Mirrored',
                 'RAIDType': 'RAID1',
                 'CapacityBytes': 536870912000,
                 'Links': {
@@ -631,8 +624,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -724,8 +715,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload1 = {
-                'Encrypted': False,
-                'VolumeType': 'SpannedMirrors',
                 'RAIDType': 'RAID10',
                 'CapacityBytes': 53687091200,
                 'Links': {
@@ -735,11 +724,10 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                         {'@odata.id': pre + self.drive_id3},
                         {'@odata.id': pre + self.drive_id4}
                     ]
-                }
+                },
+                'Name': 'root_volume'
             }
             expected_payload2 = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -748,7 +736,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                         {'@odata.id': pre + self.drive_id3},
                         {'@odata.id': pre + self.drive_id4}
                     ]
-                }
+                },
+                'Name': 'data_volume'
             }
             self.assertEqual(
                 self.mock_storage.volumes.create.call_count, 2)
@@ -852,8 +841,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload1 = {
-                'Encrypted': False,
-                'VolumeType': 'Mirrored',
                 'RAIDType': 'RAID1',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -864,8 +851,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 }
             }
             expected_payload2 = {
-                'Encrypted': False,
-                'VolumeType': 'NonRedundant',
                 'RAIDType': 'RAID0',
                 'CapacityBytes': 536870912000,
                 'Links': {
@@ -975,8 +960,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             task.driver.raid.create_configuration(task)
             pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
             expected_payload1 = {
-                'Encrypted': False,
-                'VolumeType': 'StripedWithParity',
                 'RAIDType': 'RAID5',
                 'CapacityBytes': 107374182400,
                 'Links': {
@@ -988,8 +971,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 }
             }
             expected_payload2 = {
-                'Encrypted': False,
-                'VolumeType': 'Mirrored',
                 'RAIDType': 'RAID1',
                 'CapacityBytes': 536870912000,
                 'Links': {
@@ -1038,7 +1019,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         mock_volumes = []
         for i in ["1", "2"]:
             mock_volumes.append(_mock_volume(
-                i, volume_type='Mirrored', raid_type=sushy.RAIDType.RAID1))
+                i, raid_type=sushy.RAIDType.RAID1))
         op_apply_time_support = mock.MagicMock()
         op_apply_time_support.mapped_supported_values = [
             sushy.APPLY_TIME_IMMEDIATE, sushy.APPLY_TIME_ON_RESET]
@@ -1092,7 +1073,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         mock_volumes = []
         for i in ["1", "2"]:
             mock_volumes.append(_mock_volume(
-                i, volume_type='Mirrored', raid_type=sushy.RAIDType.RAID1))
+                i, raid_type=sushy.RAIDType.RAID1))
         op_apply_time_support = mock.MagicMock()
         op_apply_time_support.mapped_supported_values = [
             sushy.APPLY_TIME_ON_RESET]
@@ -1123,7 +1104,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 task.node, reboot=True, skip_current_step=True, polling=True)
             mock_get_async_step_return_state.assert_called_once_with(
                 task.node)
-            mock_node_power_action.assert_called_once_with(task, states.REBOOT)
+            mock_node_power_action.assert_called_once_with(
+                task, states.REBOOT, None)
             mock_build_agent_options.assert_called_once_with(task.node)
             self.assertEqual(mock_prepare_ramdisk.call_count, 1)
             self.assertEqual(
@@ -1146,16 +1128,10 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         capacity_bytes = 53739520000
         pre = '/redfish/v1/Systems/1/Storage/1/Drives/'
         expected_payload = {
-            'Encrypted': False,
-            'VolumeType': 'Mirrored',
             'RAIDType': 'RAID1',
             'CapacityBytes': capacity_bytes,
             'Links': {
-                'Drives': [
-                    {
-                        '@odata.id': pre + drive_id
-                    }
-                ]
+                'Drives': [{'@odata.id': pre + drive_id}]
             }
         }
         with task_manager.acquire(self.context, self.node.uuid,
